/********************************************************************************
** Form generated from reading UI file 'mainwindow.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QListView>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include <ktoolbar.h>
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QWidget *centralwidget;
    QVBoxLayout *verticalLayout_4;
    QStackedWidget *stackedWidget;
    QWidget *displayPage;
    QVBoxLayout *verticalLayout_6;
    QTabWidget *documents;
    QWidget *openPage;
    QVBoxLayout *verticalLayout_7;
    QSpacerItem *verticalSpacer;
    QWidget *widget;
    QHBoxLayout *horizontalLayout;
    QToolButton *openFile;
    QSpacerItem *verticalSpacer_2;
    QMenuBar *menubar;
    QStatusBar *statusbar;
    QDockWidget *dataTreeDock;
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout;
    KLineEdit *filterDataTree;
    QTreeView *dataTreeView;
    QDockWidget *allocatorDock;
    QWidget *dockWidgetContents_2;
    QVBoxLayout *verticalLayout_5;
    KToolBar *dockMenuBar;
    QListView *allocatorView;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName("MainWindow");
        MainWindow->resize(800, 600);
        centralwidget = new QWidget(MainWindow);
        centralwidget->setObjectName("centralwidget");
        verticalLayout_4 = new QVBoxLayout(centralwidget);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        stackedWidget = new QStackedWidget(centralwidget);
        stackedWidget->setObjectName("stackedWidget");
        displayPage = new QWidget();
        displayPage->setObjectName("displayPage");
        verticalLayout_6 = new QVBoxLayout(displayPage);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(0, 0, 0, 0);
        documents = new QTabWidget(displayPage);
        documents->setObjectName("documents");
        documents->setDocumentMode(true);

        verticalLayout_6->addWidget(documents);

        stackedWidget->addWidget(displayPage);
        openPage = new QWidget();
        openPage->setObjectName("openPage");
        verticalLayout_7 = new QVBoxLayout(openPage);
        verticalLayout_7->setObjectName("verticalLayout_7");
        verticalSpacer = new QSpacerItem(20, 252, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer);

        widget = new QWidget(openPage);
        widget->setObjectName("widget");
        horizontalLayout = new QHBoxLayout(widget);
        horizontalLayout->setObjectName("horizontalLayout");
        openFile = new QToolButton(widget);
        openFile->setObjectName("openFile");
        openFile->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);

        horizontalLayout->addWidget(openFile);


        verticalLayout_7->addWidget(widget);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer_2);

        stackedWidget->addWidget(openPage);

        verticalLayout_4->addWidget(stackedWidget);

        MainWindow->setCentralWidget(centralwidget);
        menubar = new QMenuBar(MainWindow);
        menubar->setObjectName("menubar");
        menubar->setGeometry(QRect(0, 0, 800, 19));
        MainWindow->setMenuBar(menubar);
        statusbar = new QStatusBar(MainWindow);
        statusbar->setObjectName("statusbar");
        MainWindow->setStatusBar(statusbar);
        dataTreeDock = new QDockWidget(MainWindow);
        dataTreeDock->setObjectName("dataTreeDock");
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName("dockWidgetContents");
        verticalLayout = new QVBoxLayout(dockWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        filterDataTree = new KLineEdit(dockWidgetContents);
        filterDataTree->setObjectName("filterDataTree");
        filterDataTree->setProperty("urlDropsEnabled", QVariant(false));
        filterDataTree->setProperty("showClearButton", QVariant(true));

        verticalLayout->addWidget(filterDataTree);

        dataTreeView = new QTreeView(dockWidgetContents);
        dataTreeView->setObjectName("dataTreeView");

        verticalLayout->addWidget(dataTreeView);

        dataTreeDock->setWidget(dockWidgetContents);
        MainWindow->addDockWidget(Qt::DockWidgetArea::RightDockWidgetArea, dataTreeDock);
        allocatorDock = new QDockWidget(MainWindow);
        allocatorDock->setObjectName("allocatorDock");
        dockWidgetContents_2 = new QWidget();
        dockWidgetContents_2->setObjectName("dockWidgetContents_2");
        verticalLayout_5 = new QVBoxLayout(dockWidgetContents_2);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        dockMenuBar = new KToolBar(dockWidgetContents_2);
        dockMenuBar->setObjectName("dockMenuBar");

        verticalLayout_5->addWidget(dockMenuBar);

        allocatorView = new QListView(dockWidgetContents_2);
        allocatorView->setObjectName("allocatorView");

        verticalLayout_5->addWidget(allocatorView);

        allocatorDock->setWidget(dockWidgetContents_2);
        MainWindow->addDockWidget(Qt::DockWidgetArea::RightDockWidgetArea, allocatorDock);

        retranslateUi(MainWindow);

        stackedWidget->setCurrentIndex(1);
        documents->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        openFile->setText(tr2i18n("...", nullptr));
        dataTreeDock->setWindowTitle(tr2i18n("Massif Data", nullptr));
        filterDataTree->setProperty("clickMessage", QVariant(tr2i18n("filter", nullptr)));
        allocatorDock->setWindowTitle(tr2i18n("Custom Allocators", nullptr));
        (void)MainWindow;
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // MAINWINDOW_H

