/********************************************************************************
** Form generated from reading UI file 'config.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIG_H
#define UI_CONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_Config
{
public:
    QFormLayout *formLayout_2;
    QLabel *costLabel;
    QSpinBox *kcfg_PrettyCostPrecision;
    QLabel *label;
    QCheckBox *kcfg_ShortenTemplates;
    QGroupBox *groupBox;
    QFormLayout *formLayout;
    QLabel *legendPositionLabel;
    QComboBox *kcfg_LegendPosition;
    QLabel *legendAlignmentLabel;
    QComboBox *kcfg_LegendAlignment;
    QLabel *legendFontSizeLabel;
    QSpinBox *kcfg_LegendFontSize;

    void setupUi(QWidget *Config)
    {
        if (Config->objectName().isEmpty())
            Config->setObjectName("Config");
        Config->resize(283, 216);
        formLayout_2 = new QFormLayout(Config);
        formLayout_2->setObjectName("formLayout_2");
        costLabel = new QLabel(Config);
        costLabel->setObjectName("costLabel");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, costLabel);

        kcfg_PrettyCostPrecision = new QSpinBox(Config);
        kcfg_PrettyCostPrecision->setObjectName("kcfg_PrettyCostPrecision");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, kcfg_PrettyCostPrecision);

        label = new QLabel(Config);
        label->setObjectName("label");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, label);

        kcfg_ShortenTemplates = new QCheckBox(Config);
        kcfg_ShortenTemplates->setObjectName("kcfg_ShortenTemplates");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, kcfg_ShortenTemplates);

        groupBox = new QGroupBox(Config);
        groupBox->setObjectName("groupBox");
        formLayout = new QFormLayout(groupBox);
        formLayout->setObjectName("formLayout");
        legendPositionLabel = new QLabel(groupBox);
        legendPositionLabel->setObjectName("legendPositionLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, legendPositionLabel);

        kcfg_LegendPosition = new QComboBox(groupBox);
        kcfg_LegendPosition->addItem(QString());
        kcfg_LegendPosition->addItem(QString());
        kcfg_LegendPosition->addItem(QString());
        kcfg_LegendPosition->addItem(QString());
        kcfg_LegendPosition->addItem(QString());
        kcfg_LegendPosition->setObjectName("kcfg_LegendPosition");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, kcfg_LegendPosition);

        legendAlignmentLabel = new QLabel(groupBox);
        legendAlignmentLabel->setObjectName("legendAlignmentLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, legendAlignmentLabel);

        kcfg_LegendAlignment = new QComboBox(groupBox);
        kcfg_LegendAlignment->addItem(QString());
        kcfg_LegendAlignment->addItem(QString());
        kcfg_LegendAlignment->addItem(QString());
        kcfg_LegendAlignment->addItem(QString());
        kcfg_LegendAlignment->addItem(QString());
        kcfg_LegendAlignment->setObjectName("kcfg_LegendAlignment");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, kcfg_LegendAlignment);

        legendFontSizeLabel = new QLabel(groupBox);
        legendFontSizeLabel->setObjectName("legendFontSizeLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, legendFontSizeLabel);

        kcfg_LegendFontSize = new QSpinBox(groupBox);
        kcfg_LegendFontSize->setObjectName("kcfg_LegendFontSize");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, kcfg_LegendFontSize);


        formLayout_2->setWidget(2, QFormLayout::ItemRole::SpanningRole, groupBox);


        retranslateUi(Config);

        QMetaObject::connectSlotsByName(Config);
    } // setupUi

    void retranslateUi(QWidget *Config)
    {
        costLabel->setText(tr2i18n("Cost Precision:", nullptr));
        label->setText(tr2i18n("Shorten Templates:", nullptr));
        groupBox->setTitle(tr2i18n("Legend", nullptr));
        legendPositionLabel->setText(tr2i18n("Position:", nullptr));
        kcfg_LegendPosition->setItemText(0, tr2i18n("North", nullptr));
        kcfg_LegendPosition->setItemText(1, tr2i18n("East", nullptr));
        kcfg_LegendPosition->setItemText(2, tr2i18n("South", nullptr));
        kcfg_LegendPosition->setItemText(3, tr2i18n("West", nullptr));
        kcfg_LegendPosition->setItemText(4, tr2i18n("Floating", nullptr));

        legendAlignmentLabel->setText(tr2i18n("Alignment:", nullptr));
        kcfg_LegendAlignment->setItemText(0, tr2i18n("Left", nullptr));
        kcfg_LegendAlignment->setItemText(1, tr2i18n("Center", nullptr));
        kcfg_LegendAlignment->setItemText(2, tr2i18n("Right", nullptr));
        kcfg_LegendAlignment->setItemText(3, tr2i18n("Top", nullptr));
        kcfg_LegendAlignment->setItemText(4, tr2i18n("Bottom", nullptr));

        legendFontSizeLabel->setText(tr2i18n("Font Size:", nullptr));
        (void)Config;
    } // retranslateUi

};

namespace Ui {
    class Config: public Ui_Config {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIG_H

